(*
 * Copyright (C) 2012 Zhang Initiative Research Unit,
 * Advance Science Institute, RIKEN
 * 2-1 Hirosawa, Wako, Saitama 351-0198, Japan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *)

module HT = Hashtbl
module MU = My_utils
module S  = String

let atof = float_of_string

let xyz_of_pdb_line l =
  try
    let xs, ys, zs = S.sub l 30 8,
                     S.sub l 38 8,
                     S.sub l 46 8 in
    (atof xs, atof ys, atof zs)
  with _ ->
    failwith ("pdb_parser.ml: xyz_of_pdb_line: cannot extract xyz from: " ^ l)

(* regexp for ligand lines in the .pqr file generated by pdb2pqr when a small
   molecule ligand is compined to a protein receptor *)
let ligand_regexp         = Str.regexp "^HETATM.*LIG L"
let atom_or_hetatm_regexp = Str.regexp "^\\(ATOM\\|HETATM\\)"

let is_ligand l         = Str.string_match ligand_regexp         l 0
let is_atom_or_hetatm l = Str.string_match atom_or_hetatm_regexp l 0

(* xyz triplet to corresponding pdb line hash table creation *)
let create_xyz_to_pdb_line f =
  MU.enforce_any_file_extension f [".pdb"];
  let xyz_to_pdb_line = HT.create 1000 in
  let insert_in_ht pdb_line =
    HT.add xyz_to_pdb_line (xyz_of_pdb_line pdb_line) pdb_line
  in
  MU.iter_on_some_lines_of_file is_atom_or_hetatm insert_in_ht f;
  xyz_to_pdb_line
